/* $Id: clock.c,v 1.6 2007/09/02 23:44:55 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "module.h"
#include "gpanel.h"
#include "language.h"

extern gboolean debug;		/* see, gpanel.c */
extern const char *Program;	/* .. */

static GtkWidget *calendar;

/*
 * (private) show_calendar
 * (private) show_time
 */
static gboolean
show_calendar (GtkWidget *widget, GlobalPanel *panel)
{
  g_return_val_if_fail (GTK_IS_WIDGET(widget), FALSE);

  if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(widget)))
    gtk_widget_show (calendar);
  else
    gtk_widget_hide (calendar);

  return FALSE;
} /* </show_calendar> */

static gboolean
show_time (PanelClock *clock)
{
  char text[FILENAME_MAX];
  time_t now = time(NULL);

  strftime(text, FILENAME_MAX, clock->format, localtime(&now));
  gtk_label_set_markup (GTK_LABEL(clock->display), text);

  return TRUE;
} /* </show_time> */

/*
 * calendar_new
 */
static GtkWidget *
calendar_new (Modulus *self, GlobalPanel *panel)
{
  GtkWidget *inside, *window;
  gint xsize, ysize;
  gint xpos, ypos;

  gint width  = gdk_screen_width ();
  gint height = gdk_screen_height ();

  /* Create a popup window with a frame. */
  window = gtk_window_new (GTK_WINDOW_POPUP);

  inside = gtk_calendar_new ();
  gtk_container_add (GTK_CONTAINER(window), inside);
  gtk_widget_show (inside);

  gtk_widget_realize (window);
  gtk_window_get_size (GTK_WINDOW(window), &xsize, &ysize);

  if (panel->orientation == GTK_ORIENTATION_HORIZONTAL) {
    if (self->place == PACK_START)
      xpos = panel->spos + xsize;
    else
      xpos = panel->epos - xsize;

    if (panel->place == GTK_POS_BOTTOM)
      ypos = height - panel->height - panel->margin - ysize - 2;
    else
      ypos = panel->height + panel->margin;
  }
  else {
    if (self->place == PACK_START)
      ypos = panel->spos + panel->margin + ysize;
    else
      ypos = panel->epos - panel->margin - ysize;

    if (panel->place == GTK_POS_RIGHT)
      xpos = width - panel->width - panel->margin - xsize;
    else
      xpos = panel->width + panel->margin;
  }
  gtk_widget_set_uposition (GTK_WIDGET(window), xpos, ypos);

  return window;
} /* </calendar_new> */

/*
 * Required methods
 */
void
module_init (Modulus *self, GlobalPanel *panel)
{
  self->name    = "clock";
  self->release = "0.6";
  self->place   = PACK_END;
  self->space   = MODULI_SPACE_TASKBAR;
} /* </module_init> */

/*
 * Optional methods
 */
void
module_open (Modulus *self, GlobalPanel *panel)
{
  GtkWidget *button, *display;
  PanelClock *clock = panel->clock;

  /* construct the user interface */
  button = gtk_toggle_button_new();
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT(button), "toggled",
                    G_CALLBACK (show_calendar), panel);

  display = clock->display = gtk_label_new ("00:00");
  gtk_container_add (GTK_CONTAINER(button), display);
  gtk_widget_show (display);

  /* create the interface for the calendar, before updating pack position */
  calendar = calendar_new (self, panel);

  /* adjust GlobalPanel pack position */
  panel_update_pack_position(panel, self->place, display, 0);

  /* update the display every 'interval' seconds */
  g_timeout_add (1000 * clock->interval, show_time, clock);
  show_time (clock);

  self->widget = button;
} /* module_open */

void
module_close (Modulus *self, GlobalPanel *panel)
{
  gtk_widget_destroy (self->widget);
} /* module_close */
