/* $Id: showdesktop.c,v 1.3 2007/09/02 23:44:55 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "module.h"
#include "gpanel.h"

/*
 * (private) toggle_showing_desktop
 */
static gboolean
toggle_showing_desktop (GtkWidget *widget, GlobalPanel *panel)
{
  static GList *clients = NULL;
  PanelWnck *wnck = panel->wnck;
  WnckScreen *screen = wnck->screen;
  Display *display = GDK_WINDOW_XDISPLAY( GDK_ROOT_PARENT() );
  Window root = DefaultRootWindow(display);
  Window *children, parent, xid;
  XWindowAttributes xwa;

  guint count, idx;
  char **argv = NULL;
  int argc;

  wnck->showdesktop = !wnck->showdesktop;
  wnck_screen_toggle_showing_desktop (screen, wnck->showdesktop);

  if (wnck->showdesktop) {
    GList *iter;

    for (iter = clients; iter != NULL; iter = iter->next)
      XMapWindow(display, (Window)iter->data);

    g_list_free (clients);
    clients = NULL;
  }
  else {
    if (XQueryTree(display, root, &root, &parent, &children, &count))
      for (idx = 0; idx < count; idx++)
        if ((xid = XmuClientWindow (display, children[idx])) != None)
          if (XGetCommand(display, xid, &argv, &argc))
            if (XGetWindowAttributes(display, xid, &xwa))
              if (xwa.map_state == IsViewable) {
                clients = g_list_append(clients, xid);
                XUnmapWindow(display, xid);
              }
  }
  return TRUE;
} /* </toggle_showing_desktop> */

/*
 * Required methods
 */
void
module_init (Modulus *self, GlobalPanel *panel)
{
  self->name    = "showdesktop";
  self->release = "1.0";
  self->place   = PACK_START;
  self->space   = MODULI_SPACE_TASKBAR;
} /* </module_init> */

/*
 * Optional methods
 */
void
module_open (Modulus *self, GlobalPanel *panel)
{
  GtkWidget *button;
  GtkWidget *widget = NULL;
  PanelIcons *icons = panel->icons;

  /* construct the user interface */
  button = gtk_toggle_button_new ();
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT (button), "toggled",
                    G_CALLBACK (toggle_showing_desktop), panel);

  if (self->icon) {
    const gchar *file = file_path_find (icons->path, self->icon);

    if (file != NULL) {
      widget = image_new_from_file_scaled (file, icons->size, icons->size);
      panel_update_pack_position(panel, self->place, NULL, icons->size);
    }
  }

  if (widget == NULL) {
    widget = gtk_label_new (self->name);
    panel_update_pack_position(panel, self->place, widget, 0);
  }

  gtk_container_add (GTK_CONTAINER (button), widget);
  gtk_widget_show (widget);

  self->widget = button;
} /* module_open */

void
module_close (Modulus *self, GlobalPanel *panel)
{
  gtk_widget_destroy (self->widget);
} /* module_close */
