/* $Id: splash.c,v 1.3 2007/08/28 18:20:35 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "module.h"
#include "gpanel.h"
#include "docklet.h"

/*
 * (private) destruct
 */
static gboolean
destruct (Modulus *self)
{
  gtk_widget_destroy (self->widget);
  return FALSE;
} /* </destruct> */

/*
 * module_interface - contructs the user interface
 */
GtkWidget *
module_interface (Modulus *self, GlobalPanel *panel)
{
  GError     *error = NULL;
  PanelSplash *splash = panel->splash;
  const gchar *icon = file_path_find (panel->icons->path, self->icon);
  GdkPixbuf *pixbuf = gdk_pixbuf_new_from_file (icon, &error);

  gint xsize = gdk_pixbuf_get_width (pixbuf);
  gint ysize = gdk_pixbuf_get_height (pixbuf);

  gint width  = gdk_screen_width ();
  gint height = gdk_screen_height ();

  Docklet *docklet = docklet_new (GDK_WINDOW_TYPE_HINT_DOCK,
                                  xsize, ysize,
                                  (width - 3*xsize) / 2, (height - ysize) / 2,
                                  GTK_ORIENTATION_HORIZONTAL,
                                  GTK_VISIBILITY_FULL,
                                  icon,
                                  self->description,
				  splash->fontdesc,
                                  splash->fg,
                                  splash->bg);

  GtkWidget *widget = docklet->window;
  g_object_unref (pixbuf);

  /* We do not want drag-n-drop Docklet behavior. */
  g_signal_handler_disconnect (G_OBJECT (widget), docklet->buttonHandler);
  g_signal_handler_disconnect (G_OBJECT (widget), docklet->windowHandler);

  /* Keep the window above all others, and change cursor. */
  gtk_window_set_keep_above (GTK_WINDOW(widget), TRUE);
  docklet_set_cursor (docklet, GDK_CIRCLE);

  if (splash->expire > 0)	/* self destruct in 'expire' seconds */
    g_timeout_add (1000 * splash->expire, destruct, self);

  return widget;
} /* </module_interface> */

/*
 * Required methods
 */
void
module_init (Modulus *self, GlobalPanel *panel)
{
  self->name    = "splash";
  self->release = "0.2";
  self->place   = SCREEN;
  self->space   = MODULI_SPACE_TASKBAR;
  self->icon    = "welcome.png";
} /* </module_init> */

/*
 * Optional methods
 */
void
module_open (Modulus *self, GlobalPanel *panel)
{
  if (self->description == NULL) {
    if (g_file_test("/etc/lsb-release", G_FILE_TEST_EXISTS)) {
      FILE *stream;
      gchar text[FILENAME_MAX];

      stream = popen("lsb_release -d", "r");
      fgets(text, FILENAME_MAX, stream);
      pclose(stream);

      self->description = g_strdup(&text[strlen("Description:\t")]);
    }
  }

  /* Without self->description return. */
  if (self->description == NULL) {
    self->widget = NULL;
    return;
  }

  self->widget = module_interface (self, panel);
} /* module_open */

void
module_close (Modulus *self, GlobalPanel *panel)
{
  gtk_widget_destroy (self->widget);
} /* module_close */
