/* $Id: desktop.c,v 1.11 2007/09/30 15:41:11 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpanel.h"
#include "docklet.h"
#include "language.h"

extern GlobalPanel *global;				/* .... */
extern const char *Program, *Release, *Information;	/* see, gpanel.c */
extern gboolean debug;					/* .... */

static char *DesktopMenu =
"<?xml version=\"1.0\" encoding=\"utf-16\"?>\n"
"<menu name=\"DesktopSettings\" iconsize=\"16\">\n"
" <options>\n"
"  <item name=\"Open\">\n"
"   <icon>launch.png</icon>\n"
"   <exec type=\"method\">shortcut:open</exec>\n"
"  </item>\n"
"  <item name=\"Preferences\">\n"
"   <icon>editicon.png</icon>\n"
"   <exec type=\"method\">shortcut:edit</exec>\n"
"  </item>\n"
"  <item name=\"Delete\">\n"
"   <icon>editcut.png</icon>\n"
"   <exec type=\"method\">shortcut:delete</exec>\n"
"  </item>\n"
" </options>\n"
"</menu>\n";

static gchar *ActionHintDelete = "Press the apply button to delete.";
static gchar *ActionHintEdit   = "Click on the icon to change it.";

static gchar *DefaultIconExec = "exec.png";	/* default icon for programs */
static gchar *DefaultIconURL  = "earth.png";	/* default icon for URLs */

/*
 * (private) desktop_move - change the position of a desktop shortcut
 */
static void
desktop_move (ConfigurationNode *node, gint xpos, gint ypos)
{
  ConfigurationNode *item;
  gchar *name = configuration_attrib(node, "name");

  if ((item = configuration_find(node, "xpos")) != NULL) {
    g_free (item->element);
    item->element = g_strdup_printf("%d", xpos);
  }
  if ((item = configuration_find(node, "ypos")) != NULL) {
    g_free (item->element);
    item->element = g_strdup_printf("%d", ypos);
  }
  if (debug) printf("%s moved to (%d,%d)\n", name, xpos, ypos);
} /* </desktop_move> */

/*
 * (private) desktop_filer_apply
 * (private) desktop_filer_cancel
 */
static gboolean
desktop_filer_apply (GtkWidget *button, GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;

  gtk_widget_hide (desktop->filer);
  gtk_button_set_image (GTK_BUTTON(desktop->iconview),
			gtk_image_new_from_pixbuf (desktop->render));

  return TRUE;
} /* </desktop_filer_apply> */

static gboolean
desktop_filer_cancel (GtkWidget *button, GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;
  gtk_widget_hide (desktop->filer);
  return TRUE;
} /* </desktop_filer_cancel> */

/*
 * (private) desktop_filer_refresh
 * (private) desktop_filer_repaint
 */
gboolean
desktop_filer_refresh (GtkWidget *canvas,
                       GdkEventExpose *event,
                       GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;
  redraw_pixbuf (canvas, desktop->render);
  return TRUE;
} /* </desktop_filer_refresh> */

static gboolean
desktop_filer_repaint (FileChooserDatum *datum)
{
  GlobalPanel *self = datum->user;
  PanelDesktop *desktop = self->desktop;
  const gchar *file = datum->file;
  struct stat info;

  if (lstat(file, &info) == 0 && S_ISREG(info.st_mode)) {
    gint iconsize = desktop->iconsize;

    desktop->render = pixbuf_new_from_file_scaled (file, iconsize, iconsize);
    desktop_filer_refresh(desktop->canvas, NULL, self);
  }

  return TRUE;
} /* </desktop_filer_repaint> */

/*
 * (private) desktop_setting_apply
 * (private) desktop_setting_cancel
 */
static gboolean
desktop_setting_apply (GtkWidget *button, GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;
  DesktopAction action = desktop->action;
  Docklet *docklet = desktop->shortcut;

  /* Dismiss the desktop->window interface. */
  gtk_widget_hide (desktop->window);
  desktop->activate = TRUE;

  /* Handle the user specified action. */
  if (action == DESKTOP_SHORTCUT_DELETE) {
    configuration_remove (desktop->node);   /* remove from configuration */
    gtk_widget_destroy (docklet->window);   /* remove from screen display */
    g_object_ref_sink (docklet);	    /* clean-up memory */
    g_object_unref (docklet);
  }
  else {
    FileChooser *chooser = desktop->chooser;
    const gchar *iconame = gtk_entry_get_text (GTK_ENTRY(chooser->name));
    const gchar *icopath = filechooser_get_selected_name (chooser);

    const gchar *text = gtk_entry_get_text (GTK_ENTRY(desktop->name));
    const gchar *run  = gtk_entry_get_text (GTK_ENTRY(desktop->exec));

    ConfigurationNode *config = configuration_find (self->config, "desktop");
    gchar *font = configuration_attrib(config, "font");

    if (file_path_find (self->icons->path, iconame) == NULL)
      iconame = icopath;

    if (action == DESKTOP_SHORTCUT_EDIT) {
      ConfigurationNode *node = desktop->node;
      ConfigurationAttrib *attrib = node->attrib;

      ConfigurationNode *exec = configuration_find (node, "exec");
      ConfigurationNode *icon = configuration_find (node, "icon");

      g_free (icon->element);
      icon->element = g_strdup (iconame);

      g_free (exec->element);
      exec->element = g_strdup (run);

      /* DANGER! will break unless we only have <item name="{value}"> */
      g_free (attrib->value);
      attrib->value = g_strdup (text);

      docklet_update (docklet, icopath, text);	/* update screen display */
    }
    else if (action == DESKTOP_SHORTCUT_CREATE) {
      guint iconsize = desktop->iconsize;
      gint xpos = 100;
      gint ypos = 100;

      gchar *stub =
"<?xml version=\"1.0\" encoding=\"utf-16\"?>\n"
"<item name=\"%s\">\n"
" <icon>%s</icon>\n"
" <exec>%s</exec>\n"
" <xpos>%d</xpos>\n"
" <ypos>%d</ypos>\n"
"</item>\n";

      gchar *spec = g_strdup_printf(stub, text, iconame, run, xpos, ypos);
      ConfigurationNode *chain = configuration_read (spec, NULL, TRUE);

      ConfigurationNode *mark = configuration_find (config, "/desktop");
      ConfigurationNode *item = configuration_find (chain, "item");
      Docklet *docklet;

      configuration_insert (mark->back, item, TRUE); /* insert new node */
      g_free (spec);

      docklet = desktop_shortcut (self, item,	     /* new screen visual */
                                  iconsize, iconsize,
                                  xpos, ypos,
                                  icopath, text, font,
                                  NULL, NULL);

      gtk_widget_show (docklet->window);
    }
  }
  return TRUE;
} /* </desktop_setting_apply> */

static gboolean
desktop_setting_cancel (GtkWidget *button, GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;

  gtk_widget_hide (desktop->window);
  desktop->activate = TRUE;

  return TRUE;
} /* </desktop_setting_cancel> */

/*
 * (private) desktop_setting_iconview
 */
static gboolean
desktop_setting_iconview (GtkWidget *button, GlobalPanel *self)
{
  PanelDesktop *desktop = self->desktop;
  gtk_widget_show (desktop->filer);
  return TRUE;
} /* </desktop_setting_iconview> */

/*
 * desktop_new - object for creating and editing desktop shortcuts
 */
PanelDesktop *
desktop_new (GlobalPanel *self, gint iconsize)
{
  GtkWidget *window, *frame, *layout, *layer;
  GtkWidget *button, *entry, *label, *split;

  ConfigurationNode *config = configuration_read (DesktopMenu, NULL, TRUE);
  PanelDesktop *desktop = g_new0 (PanelDesktop, 1);
  PanelIcons *icons = self->icons;

  FileChooser *chooser;
  gchar dirname[FILENAME_MAX];
  gchar *scan;

  desktop->activate = TRUE;
  desktop->iconsize = iconsize;
  desktop->menu = menu_options_config (config, self);

  /* Create a sticky window with a frame. */
  window = desktop->window = sticky_window_new (GDK_WINDOW_TYPE_HINT_NORMAL,
                                                        280, 180, 100, 100);
  gtk_window_set_keep_above (GTK_WINDOW(window), TRUE);

  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_container_add (GTK_CONTAINER(window), frame);
  gtk_widget_show (frame);

  /* Create the inside window layout. */
  layout = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(frame), layout);
  gtk_widget_show (layout);

  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layout), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  /* Top portion has the icon and FileChooser */
  layer = gtk_hbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(layout), layer);
  gtk_widget_show (layer);

  /* Brute force addition of horizontal spacing. */
  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layer), label, FALSE, FALSE, 5);
  gtk_widget_show (label);

  label = gtk_label_new ("");
  gtk_box_pack_end (GTK_BOX(layer), label, FALSE, FALSE, 5);
  gtk_widget_show (label);

  button = desktop->iconview = gtk_button_new ();
  gtk_box_pack_start (GTK_BOX(layer), button, FALSE, FALSE, 2);
  gtk_widget_show (button);

  label = desktop->hint = gtk_label_new (_(ActionHintEdit));
  gtk_box_pack_start (GTK_BOX(layer), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(desktop_setting_iconview), self);

  /* Middle portion has text entries for name and program. */
  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layout), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  layer = gtk_hbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(layout), layer);
  gtk_widget_show (layer);

  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layer), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  label = gtk_label_new (_("Name:"));
  gtk_box_pack_start (GTK_BOX(layer), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  entry = desktop->name = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX(layer), entry, FALSE, FALSE, 2);
  gtk_widget_show (entry);

  label = gtk_label_new ("");
  gtk_box_pack_end (GTK_BOX(layer), label, FALSE, FALSE, 5);
  gtk_widget_show (label);

  /* Box for the exec item (program, url, ..) */
  layer = gtk_hbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(layout), layer);
  gtk_widget_show (layer);

  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layer), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  label = gtk_label_new (_("Program:"));
  gtk_box_pack_start (GTK_BOX(layer), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  entry = desktop->exec = gtk_entry_new ();
  gtk_box_pack_start (GTK_BOX(layer), entry, FALSE, FALSE, 2);
  gtk_widget_show (entry);

  label = gtk_label_new ("");
  gtk_box_pack_end (GTK_BOX(layer), label, FALSE, FALSE, 5);
  gtk_widget_show (label);

  label = gtk_label_new ("");
  gtk_box_pack_start (GTK_BOX(layout), label, FALSE, FALSE, 0);
  gtk_widget_show (label);

  /* Bottom portion has buttons for cancel and apply. */
  layer = gtk_hbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(layout), layer);
  gtk_widget_show (layer);

  button = xpm_button(ICON_APPLY, _L(LABEL_APPLY));
  gtk_box_pack_end (GTK_BOX(layer), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(desktop_setting_apply), self);
  gtk_widget_show (button);

  button = xpm_button(ICON_CANCEL, _L(LABEL_CANCEL));
  gtk_box_pack_end (GTK_BOX(layer), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(desktop_setting_cancel), self);
  gtk_widget_show (button);

  /* Save the initial default image path. */
  desktop->icon = g_strdup (file_path_find (icons->path, DefaultIconExec));
  gtk_entry_set_text (GTK_ENTRY(desktop->name), DefaultIconExec);

  /* Create popup window for choosing a different icon. */
  window = desktop->filer = gtk_window_new (GTK_WINDOW_POPUP);
  gtk_widget_set_uposition (GTK_WIDGET(window), 120, 120);

  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_container_add (GTK_CONTAINER(window), frame);
  gtk_widget_show (frame);

  /* Split window in two areas. */
  split = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(frame), split);
  gtk_widget_show (split);

  layout = gtk_hbox_new (FALSE, 1);
  gtk_box_pack_start (GTK_BOX(split), layout, FALSE, FALSE, 0);
  gtk_widget_show (layout);

  /* Create an instance of FileChooser using desktop->icon dirname value. */
  desktop->render = pixbuf_new_from_file_scaled (desktop->icon,
                                                 iconsize,
                                                 iconsize);
  strcpy(dirname, desktop->icon);
  scan = strrchr(dirname, '/');
  *scan = (gchar)0;

  chooser = desktop->chooser = filechooser_new (dirname);
  filechooser_set_callback (chooser, desktop_filer_repaint, self);
  gtk_entry_set_text (GTK_ENTRY(chooser->name), DefaultIconExec);
  chooser->clearname  = TRUE;	/* clear file name on directory change */

  layer = filechooser_layout (chooser);
  gtk_widget_set_size_request (layer, 380, 4*iconsize);
  gtk_box_pack_start (GTK_BOX(layout), layer, TRUE, TRUE, 5);
  /* gtk_widget_hide (chooser->namebox); */
  gtk_widget_show (layer);

  layer = desktop->canvas = gtk_drawing_area_new ();
  gtk_widget_set_size_request (layer, iconsize, iconsize);
  gtk_box_pack_start (GTK_BOX(layout), layer, TRUE, TRUE, 5);
  gtk_widget_show (layer);

  g_signal_connect (G_OBJECT (layer), "expose_event",
                    G_CALLBACK (desktop_filer_refresh), self);

  /* Add the APPLY and CANCEL buttons. */
  layout = gtk_hbox_new (FALSE, 1);
  gtk_box_pack_start (GTK_BOX(split), layout, FALSE, FALSE, 0);
  gtk_widget_show (layout);

  button = xpm_button(ICON_APPLY, _L(LABEL_APPLY));
  gtk_box_pack_end (GTK_BOX(layout), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(desktop_filer_apply), self);
  gtk_widget_show (button);

  button = xpm_button(ICON_CANCEL, _L(LABEL_CANCEL));
  gtk_box_pack_end (GTK_BOX(layout), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);
  gtk_widget_show (button);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(desktop_filer_cancel), self);

  return desktop;
} /* desktop_new */

/*
 * (private) desktop_menu - present menu of desktop shortcut
 * (private) desktop_settings - change the settings of desktop shortcut
 */
void
desktop_menu (ConfigurationNode *node, Docklet *docklet)
{
  GlobalPanel *self = node->data;	 /* cannot be NULL */
  PanelDesktop *desktop = self->desktop;

  ConfigurationNode *exec = configuration_find (node, "exec");
  ConfigurationNode *icon = configuration_find (node, "icon");

  gchar *name = configuration_attrib (node, "name");
  const gchar *file = file_path_find (self->icons->path, icon->element);
  gint iconsize = desktop->iconsize;

  desktop->render = pixbuf_new_from_file_scaled (file, iconsize, iconsize);
  gtk_button_set_image (GTK_BUTTON(desktop->iconview),
                        gtk_image_new_from_pixbuf (desktop->render));

  gtk_entry_set_text (GTK_ENTRY(desktop->name), name);
  gtk_entry_set_text (GTK_ENTRY(desktop->exec), exec->element);
  gtk_entry_set_text (GTK_ENTRY(desktop->chooser->name), icon->element);

  desktop->node = node;		/* configuration node data */
  desktop->shortcut = docklet;	/* screen representation */

  gtk_menu_popup (GTK_MENU(desktop->menu), NULL, NULL, NULL, NULL, 0, 0);
} /* </desktop_menu> */

void
desktop_settings (ConfigurationNode *node, DesktopAction act)
{
  GlobalPanel *panel = node->data;	/* cannot be NULL */
  PanelDesktop *desktop = panel->desktop;
  
  if (desktop->activate) {
    GtkWidget *iconview = desktop->iconview;

    if (act == DESKTOP_SHORTCUT_CREATE) {
      const gchar *file = desktop->icon;
      gint iconsize = desktop->iconsize;

      desktop->render = pixbuf_new_from_file_scaled (file, iconsize, iconsize);
      gtk_button_set_image (GTK_BUTTON(iconview),
                            gtk_image_new_from_pixbuf (desktop->render));

      gtk_entry_set_text (GTK_ENTRY(desktop->name), "");
      gtk_entry_set_text (GTK_ENTRY(desktop->exec), "");
      gtk_entry_set_text (GTK_ENTRY(desktop->chooser->name), DefaultIconExec);

      desktop->shortcut = NULL;	/* screen representation */
    }
    else if (act == DESKTOP_SHORTCUT_DELETE) {
      gtk_button_set_relief (GTK_BUTTON(iconview), GTK_RELIEF_NONE);
      gtk_widget_set_sensitive (GTK_WIDGET(iconview), FALSE);

      gtk_entry_set_editable (GTK_ENTRY(desktop->name), FALSE);
      gtk_entry_set_editable (GTK_ENTRY(desktop->exec), FALSE);

      gtk_label_set_text (GTK_LABEL(desktop->hint), _(ActionHintDelete));
    }
    else if (act == DESKTOP_SHORTCUT_EDIT) {
      gtk_button_set_relief (GTK_BUTTON(iconview), GTK_RELIEF_NORMAL);
      gtk_widget_set_sensitive (GTK_WIDGET(iconview), TRUE);

      gtk_entry_set_editable (GTK_ENTRY(desktop->name), TRUE);
      gtk_entry_set_editable (GTK_ENTRY(desktop->exec), TRUE);

      gtk_label_set_text (GTK_LABEL(desktop->hint), _(ActionHintEdit));
    }

    if (act == DESKTOP_SHORTCUT_OPEN) {
      ConfigurationNode *exec = configuration_find (desktop->node, "exec");
      spawn (exec, panel);
    }
    else {
      gtk_widget_show (desktop->window);  /* show manager window */
      desktop->activate = FALSE;
    }
    desktop->action = act; /* action applied on shortcut */
  }
} /* </desktop_settings> */

/*
 * (private) reload - our callback agent for Docklet events
 */
static gboolean
reload (DockletDatum *datum)
{
  ConfigurationNode *node = (ConfigurationNode *)datum->user;
  GtkWidget *widget = datum->self->window;
  GdkEvent *event = datum->event;

  if (event->type == GDK_BUTTON_PRESS || event->type == GDK_2BUTTON_PRESS) {
    GdkEventButton *mouse = (GdkEventButton *)event;

    if (mouse->button == 1)
      executer (widget, node);
    else if (mouse->button == 3)
      desktop_menu (node, datum->self);
  }
  else if (event->type == GDK_CONFIGURE) {
    GdkEventConfigure *box = (GdkEventConfigure *)event;
    desktop_move (node, box->x, box->y);
  }
  return FALSE;
} /* </reload> */

/*
 * desktop_config - add gtk_event_box_new() to the interface layout
 */
void
desktop_config (GlobalPanel *self)
{
  Docklet *docklet;	/* a desktop item is a Docklet instance */

  ConfigurationNode *node = configuration_find (self->config, "desktop");
  ConfigurationNode *mark = configuration_find (node, "/desktop");
  ConfigurationNode *item;

  PanelIcons *icons = self->icons;
  guint depth = node->depth + 1;

  gchar *xpos, *ypos;
  const gchar *icon;
  guint iconsize;
  gchar *value;
  gchar *name;
  gchar *font;

  /* Prefer the iconsize for <desktop> */
  if ((value = configuration_attrib(node, "iconsize")) != NULL)
    iconsize = atoi(value);
  else
    iconsize = icons->size;

  /* See if font attribute is set on <desktop> */
  font = configuration_attrib(node, "font");

  for (; node != mark && node != NULL; node = node->next) {
    if (node->depth == depth && node->type != XML_READER_TYPE_END_ELEMENT) {
      name = configuration_attrib(node, "name");

      if ((item = configuration_find(node, "icon")) != NULL)
        icon = file_path_find (icons->path, item->element);
      else
        icon = NULL;

      /* Must have at least an icon or a name. */
      if (icon == NULL && name == NULL)
        continue;

      if ((item = configuration_find(node, "xpos")) != NULL)
        xpos = item->element;
      else
        xpos = NULL;

      if ((item = configuration_find(node, "ypos")) != NULL)
        ypos = item->element;
      else
        ypos = NULL;

      /* The initial position must be specified. */
      if (xpos == NULL || ypos == NULL)
        continue;
    
      /* Substitute "generic.png" if icon == NULL */
      if (icon == NULL)
        icon = file_path_find (icons->path, "blank.png");

      /* See if the Docklet has its own font, otherwise use the default. */
      if ((value = configuration_attrib(node, "font")) == NULL)
        value = font;

      /* We made it here! Create a new Docklet instance. */
      docklet = desktop_shortcut (self, node,
                                  iconsize, iconsize,
                                  atoi(xpos), atoi(ypos),
                                  icon, name, value,
                                  NULL, NULL);

      gtk_widget_show (docklet->window);
    }
  }

  /* Construct self->desktop for creating and editing desktop shortcuts. */
  self->desktop = desktop_new (self, iconsize);
} /* </desktop_config> */

/*
 * desktop_shortcut
 */
Docklet *
desktop_shortcut (GlobalPanel *self,
                  ConfigurationNode *node,
                  gint width, gint height,
                  gint xpos, gint ypos,
                  const gchar *icon,
                  const gchar *text,
                  const gchar *font,
                  GdkColor *fg,
                  GdkColor *bg)
{
  Docklet *docklet = docklet_new (GDK_WINDOW_TYPE_HINT_DOCK,
                                  width, height,
                                  xpos, ypos,
                                  GTK_ORIENTATION_VERTICAL,
                                  GTK_VISIBILITY_NONE,
                                  icon, text, font,
                                  fg, bg);
  node->data = self;			    	/* set node->data */

  docklet_set_callback (docklet, reload, node); /* specify callback */
  gtk_window_set_keep_below (GTK_WINDOW(docklet->window), TRUE);

  return docklet;
} /* </desktop_shortcut> */
