/* $Id: gcontrol.c,v 1.4 2007/08/08 14:14:26 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gcontrol.h"
#include "language.h"

static GlobalControl *global;	/* (protected) encapsulated program data */
gboolean debug = FALSE;		/* (protected) must be present */

const char *Program;		/* (public) published program name */
const char *Release;		/* (public) published program version */
const char *Information =
"is a gtk-based control panel utility.\n"
"\n"
"The program is developed by Mauro DePalma and distributed\n"
"under the terms and condition of the GNU Public License.\n";


/*
 * (private) about
 */
static gboolean
about(GtkWidget *instance, GtkWidget *parent)
{
  static gchar *message = NULL;

  if (message == NULL)
    message = g_strdup_printf("\n%s %s %s", Program, Release, Information);

  notice(parent, ICON_SNAPSHOT, message);

  return FALSE;
} /* </about> */

/*
 * clean application exit
 */
static gboolean
finis(GtkWidget *instance, gpointer data)
{
  gtk_main_quit ();
  return FALSE;
} /* </finis> */

/*
 * (private) agent
 */
static void
agent(GtkIconView *iconview, GlobalControl *self)
{
  gchar *name = iconbox_get_selected (self->iconbox, COLUMN_LABEL);
  GdkPixbuf *icon = iconbox_get_selected (self->iconbox, COLUMN_IMAGE);

  if (name) {
    char *command = NULL;

    if (strcmp(name, "System") == 0)
      command = "hardinfo";
    else if (icon == xpm_pixbuf(ICON_CHOOSER, NULL))
      command = "gdisplay";

    printf("%s::agent: %s\n", Program, name);
    if (command) system(command);
  }
} /* </agent> */

/*
 * interface - construct user interface
 */
static GtkWidget*
interface (GlobalControl *self)
{
  GtkWidget *window = self->window;
  GtkWidget *layout = gtk_vbox_new (FALSE, 0);
  GtkWidget *box, *button, *scroll, *view;
  GdkColor color = { 0, 65535, 65535, 65535 };;

  gtk_widget_modify_bg (window, GTK_STATE_NORMAL, &color); 

  box = gtk_hbox_new(FALSE, 1);
  gtk_box_pack_start(GTK_BOX (layout), box, FALSE, FALSE, 0);
  gtk_widget_show(box);

  /* Program information (about) button. */
  button = xpm_button(ICON_HELP, _L(LABEL_INFO));
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  g_signal_connect (G_OBJECT(button), "clicked", G_CALLBACK(about), window);
  gtk_box_pack_end (GTK_BOX(box), button, FALSE, TRUE, 2);
  gtk_widget_show (button);

  /* Control panel icons. */
  box = gtk_vbox_new(FALSE, 1);
  gtk_box_pack_start(GTK_BOX (layout), box, TRUE, TRUE, 0);
  gtk_widget_show(box);

  scroll = gtk_scrolled_window_new(NULL, NULL);
  gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(scroll),
                                 GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start(GTK_BOX (box), scroll, TRUE, TRUE, 5);
  gtk_widget_show(scroll);

  view = (self->iconbox)->view;
  gtk_container_add (GTK_CONTAINER(scroll), view);
  gtk_widget_show(view);

  /* CLOSE button */
  box = gtk_hbox_new(FALSE, 1);
  gtk_box_pack_start(GTK_BOX (layout), box, FALSE, FALSE, 0);
  gtk_widget_show(box);

  button = xpm_button(ICON_CLOSE, _L(LABEL_CLOSE));
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NONE);
  g_signal_connect (G_OBJECT(button), "clicked", G_CALLBACK(finis), NULL);
  /* gtk_box_pack_start(GTK_BOX (box), button, FALSE, FALSE, 0); */
  gtk_box_pack_end (GTK_BOX(box), button, FALSE, TRUE, 2);
  gtk_widget_show(button);

  return layout;
} /* </interface> */

/*
 * initialize
 */
static void
initialize (GlobalControl *self, char *imagepath)
{
  IconBox *iconbox;

  iconbox = self->iconbox = iconbox_new (GTK_ORIENTATION_VERTICAL);
  g_signal_connect (G_OBJECT(iconbox->view), "selection_changed",
                               G_CALLBACK(agent), self);

  iconbox_append (iconbox, xpm_pixbuf(ICON_CHOOSER, NULL), _L(LABEL_FILE));
  iconbox_append (iconbox, xpm_pixbuf(ICON_SNAPSHOT, NULL), _L(LABEL_CAPTURE));
  iconbox_append (iconbox, xpm_pixbuf(ICON_SETTING, NULL), "System");

  self->width  = (4 * gdk_screen_width ())  / 9;
  self->height = (3 * gdk_screen_height ()) / 9;

  global = self;   /* global program data structure */
} /* </initialize> */

/*
 * main - main program implementation
 */
int
main(int argc, char *argv[])
{
  struct _GlobalControl memory;	/* see, gcontrol.h */

  GtkWidget *window;		/* main window     */
  GtkWidget *layout;		/* user interface  */

  Program = basename(argv[0]);
  Release = "1.0.1";

  /* Initialization of the GTK */
  gtk_set_locale ();
  gtk_init (&argc, &argv);

  /* Initialize program data structure */
  initialize (&memory, (argc > 1) ? argv[1] : NULL);

  /* Create a new top level window */
  window = global->window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

  /* Set title and border width of the main window */
  gtk_window_set_title (GTK_WINDOW (window), Program);
  gtk_window_set_icon (GTK_WINDOW (window), xpm_pixbuf(ICON_CHOOSER, NULL));
  gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);
  gtk_widget_set_usize (window, global->width, global->height);
  gtk_widget_show (window);

  /* Construct the user interface */
  layout = interface (global);
  gtk_container_add (GTK_CONTAINER(window), layout);
  gtk_widget_show (layout);

  /* Set event handlers for window close/destroy */
  g_signal_connect (G_OBJECT(window), "destroy", G_CALLBACK(finis), NULL);
  g_signal_connect (G_OBJECT(window), "delete_event", G_CALLBACK(finis), NULL);

  /* Main event loop */
  gtk_main ();
    
  return Success;
} /* </main> */
