/* $Id: gpanel.c,v 1.16 2007/09/30 15:41:11 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpanel.h"
#include "language.h"

static GlobalPanel *global;	/* (protected) encapsulated program data */
gboolean debug = FALSE;		/* (protected) program debug mode */

const char *Program;		/* (public) published program name */
const char *Release;		/* (public) published program version */
const char *Information =
"is a gtk-based simple desktop panel.\n"
"\n"
"The program is developed by Mauro DePalma and distributed\n"
"under the terms and condition of the GNU Public License.\n";

const char *Schema = "panel";	/* (public) XML configuration schema */
const char *ConfigurationHeader =
"<?xml version=\"1.0\" encoding=\"utf-16\"?>\n"
"<panel>\n"
"\n"
" <!-- Automatically generated ~/.config/panel, please DO NOT edit. Manual\n"
"   changes made will be lost when the program using this file exits.  -->\n"
"\n";

static char *DefaultConfiguration =
" <place>BOTTOM</place>\n"
" <icons>\n"
"  <path>/usr/local/share/panel/pixmaps</path>\n"
"  <path>/usr/share/panel/pixmaps</path>\n"
"  <path>/usr/share/pixmaps</path>\n"
"  <path>$HOME/pixmaps</path>\n"
"  <size>22</size>\n"
" </icons>\n"
" <indent>4</indent>\n"
" <margin>2</margin>\n"
" <thickness>32</thickness>\n"
"\n"
" <modules>\n"
"  <path>/usr/local/share/panel/plugins</path>\n"
"  <path>/usr/share/panel/plugins</path>\n"
"  <path>$HOME/.config/plugins</path>\n"
"  <widget name=\"splash\" expire=\"10\" font=\"Sans 20\" fg=\"blue\">\n"
"   <icon>welcome.png</icon>\n"
"   <place>SCREEN</place>\n"
"  </widget>\n"
"  <widget name=\"menu\">\n"
"   <icon>startx.png</icon>\n"
"   <place>START</place>\n"
"  </widget>\n"
"  <widget name=\"showdesktop\">\n"
"   <icon>showdesktop.png</icon>\n"
"   <place>START</place>\n"
"  </widget>\n"
"  <widget name=\"tasklist\">\n"
"   <place>CONTAINER</place>\n"
"  </widget>\n"
"  <widget name=\"clock\" format=\"%H:%M\" interval=\"60\">\n"
"   <icon>clock.png</icon>\n"
"   <place>END</place>\n"
"  </widget>\n"
"  <widget name=\"mixer\">\n"
"   <icon>mixer.png</icon>\n"
"   <place>END</place>\n"
"  </widget>\n"
"  <widget name=\"battery\" critical=\"10\" interval=\"60\">\n"
"   <icon>battery.png</icon>\n"
"   <place>END</place>\n"
"  </widget>\n"
"  <widget name=\"pager\">\n"
"   <place>END</place>\n"
"  </widget>\n"
" </modules>\n"
"\n"
" <menu name=\"Start\" icon=\"startx.png\" iconsize=\"16\">\n"
"  <options>\n"
"   <item name=\"Console Window\">\n"
"    <icon>terminal.png</icon>\n"
"    <exec>xterm</exec>\n"
"   </item>\n"
"   <separator />\n"
"   <item name=\"Restart\">\n"
"    <icon>restart.png</icon>\n"
"    <exec type=\"method\">restart</exec>\n"
"   </item>\n"
"   <separator />\n"
"   <item name=\"Exit\">\n"
"    <icon>exit.png</icon>\n"
"    <exec type=\"method\">exit</exec>\n"
"   </item>\n"
"  </options>\n"
" </menu>\n"
"\n"
"<desktop iconsize=\"48\">\n"
" <item name=\"Recyle Bin\">\n"
"  <icon>recycle.png</icon>\n"
"   <exec>rox -d $HOME/.trash</exec>\n"
"   <xpos>6</xpos>\n"
"   <ypos>6</ypos>\n"
"  </item>\n"
"  <item name=\"Home\">\n"
"   <icon>home.png</icon>\n"
"   <exec>rox -d $HOME</exec>\n"
"   <xpos>6</xpos>\n"
"   <ypos>76</ypos>\n"
"  </item>\n"
" </desktop>\n";

/*
 * debug_configuration_cache
 */
void
debug_configuration_cache (ConfigurationNode *config, FILE* stream)
{
  if (debug)
    configuration_write (config, ConfigurationHeader, stream);
}

/*
 * (private) builtin_widgets
 */
static GList *
builtin_widgets (GlobalPanel *self)
{
  GList *list;
  GtkWidget *pager;
  GtkWidget *tasklist;
  Modulus *applet;
  PanelWnck *wnck;
  WnckScreen *screen;

  /* Initialize struct _panel_wnck data */
  wnck = self->wnck = g_new0(PanelWnck, 1);
  wnck->grouping = WNCK_TASKLIST_ALWAYS_GROUP;
  wnck->showdesktop = TRUE;
  wnck->allspaces = TRUE;
  wnck->rows = 1;

  /* Initialize WNCK */
  screen = wnck->screen = wnck_screen_get_default ();
  g_object_set_data (G_OBJECT(screen), "panel", self);
  
  /* Because the pager doesn't respond to signals at the moment */
  wnck_screen_force_update (screen);

  /* Add applications menu */
  applet = g_new0 (Modulus, 1);
  applet->name = "menu";
  applet->place = PACK_START;
  applet->space = MODULI_SPACE_TASKBAR;
  applet->widget = menu_config (applet, self);
  list = g_list_append (NULL, applet);

  /* Add pager widget from libwnck */
  pager = wnck->pager = wnck_pager_new(screen);
  wnck_pager_set_orientation (WNCK_PAGER(pager), self->orientation);
  wnck_pager_set_shadow_type (WNCK_PAGER(pager), GTK_SHADOW_IN);
  wnck_pager_set_n_rows(WNCK_PAGER(pager), wnck->rows);

  applet = g_new0 (Modulus, 1);
  applet->name = "pager";
  applet->place = PACK_START;
  applet->space = MODULI_SPACE_TASKBAR;
  applet->widget = pager;
  list = g_list_append (list, applet);
  
  /* Add tasklist widget from libwnck */
  tasklist = wnck->tasklist = wnck_tasklist_new (screen);
  wnck_tasklist_set_grouping (WNCK_TASKLIST (tasklist), wnck->grouping);
  wnck_tasklist_set_include_all_workspaces (WNCK_TASKLIST (tasklist),
                                            wnck->allspaces);
  applet = g_new0 (Modulus, 1);
  applet->name = "tasklist";
  applet->place = ADD_TO_CONTAINER;
  applet->space = MODULI_SPACE_TASKBAR;
  applet->widget = tasklist;
  list = g_list_append (list, applet);

  return list;
} /* </builtin_widgets> */

/*
 * panel_update_pack_position
 */
void
panel_update_pack_position (GlobalPanel *self,
                            ModulusPlace place,   
                            GtkWidget *label,
                            gint iconsize)
{
  gint pixels = iconsize;

  if (label != NULL) {
    GtkAllocation *allocation;
    allocation = get_label_size(label);
    pixels += allocation->width;
    g_free (allocation);
  }

  if (place == PACK_START)
    self->spos += pixels;
  else
    self->epos -= pixels;
} /* </panel_update_pack_position> */

/*
 * panel_config_orientation
 */
void
panel_config_orientation (GlobalPanel *self)
{
  gint width  = gdk_screen_width ();
  gint height = gdk_screen_height ();

  if (self->place == GTK_POS_BOTTOM || self->place == GTK_POS_TOP) {
    self->orientation = GTK_ORIENTATION_HORIZONTAL;
    self->width  = width - 2 * self->margin;
    self->height = self->thickness;
    self->xpos   = self->margin;
    self->epos   = self->width;

    if (self->place == GTK_POS_BOTTOM)
      self->ypos = height - self->thickness - 1;
    else
      self->ypos = 1;
  }
  else {
    self->orientation = GTK_ORIENTATION_VERTICAL;
    self->width  = self->thickness;
    self->height = height - 2 * self->margin;
    self->ypos   = self->margin;
    self->epos   = self->height;

    if (self->place == GTK_POS_RIGHT)
      self->xpos = width - self->thickness - 1;
    else
      self->xpos = 1;
  }

  /* Adjust start and end packing positions. */ 
  self->spos = self->margin;
  self->epos -= self->margin;
} /* </panel_config_orientation> */

/*
 * (private) panel_config_settings
 */
static void
panel_config_settings (GlobalPanel *self)
{
  ConfigurationNode *item;
  ConfigurationNode *chain;
  ConfigurationNode *config = self->config;

  PanelIcons   *icons   = g_new0 (PanelIcons, 1);
  PanelSplash  *splash  = g_new0 (PanelSplash, 1);
  PanelBattery *battery = g_new0 (PanelBattery, 1);
  PanelClock   *clock   = g_new0 (PanelClock, 1);

  gboolean gnome = FALSE;
  gchar **path = g_strsplit (getenv("PATH"), ":", FILENAME_MAX);
  gchar *attrib;
  int idx = 0;

  self->notice = NULL;		/* initialize alert notices */
  self->path = NULL;		/* initialize exec search path */

  for (idx = 0; path[idx] != NULL; idx++) {
    if (strcmp(path[idx], "/opt/gnome/bin") == 0) gnome = TRUE;
    self->path = g_list_append (self->path, g_strdup(path[idx]));
  }

  if (gnome == FALSE) {		/* add /opt/gnome/bin to self->path */
    self->path = g_list_append (self->path, g_strdup("/opt/gnome/bin"));
  }
  g_strfreev (path);

  /* Configuration for the icons. */
  icons->size = 22;
  icons->path = NULL;

  if ((chain = configuration_find (config, "icons")) != NULL) {
    gchar *path;

    if ((item = configuration_find (chain, "size")) != NULL)
      icons->size = atoi(item->element);

    if ((path = configuration_path (chain)) != NULL) {
      icons->path = g_list_append (icons->path, path);

      while (path != NULL) {
        item = configuration_find (chain, "path");  /* next <path> */

        if (item->next && (item->next)->next) {
          chain = (item->next)->next;

          if ((path = configuration_path (chain)) != NULL)
            icons->path = g_list_append (icons->path, path);
        }
      }
    }
  }
  if (icons->path == NULL)  /* fallback /usr/share/pixmaps icons path */
    icons->path = g_list_append (NULL, "/usr/share/pixmaps");

  /* Configuration for the splash screen. */
  chain = configuration_find (config, "modules");

  while ((item = configuration_find (chain, "widget")) != NULL) {
    attrib = configuration_attrib (item, "name");

    if (strcmp(attrib, "splash") == 0) {
      if ((attrib = configuration_attrib (item, "expire")) != NULL)
        splash->expire = atoi(attrib);

      if ((attrib = configuration_attrib (item, "font")) != NULL)
        splash->fontdesc = attrib;

      if ((attrib = configuration_attrib (item, "fg")) != NULL) {
        splash->fg = g_new0 (GdkColor, 1);
        gdk_color_parse (attrib, splash->fg);
        gdk_colormap_alloc_color (gdk_colormap_get_system(),
                                  splash->fg, TRUE, TRUE);
      }

      if ((attrib = configuration_attrib (item, "bg")) != NULL) {
        splash->bg = g_new0 (GdkColor, 1);
        gdk_color_parse (attrib, splash->bg);
        gdk_colormap_alloc_color (gdk_colormap_get_system(),
                                  splash->bg, TRUE, TRUE);
      }
      break;
    }
    chain = configuration_find_next (chain, "widget");
  }

  if (splash->fontdesc == NULL)
    splash->fontdesc = g_strdup ("Sans Bold 20");

  /* Configuration for the battery applet. */
  chain = configuration_find (config, "modules");

  battery->interval = 20;	/* hardcoded check interval fallback */
  battery->panel = self;

  while ((item = configuration_find (chain, "widget")) != NULL) {
    attrib = configuration_attrib (item, "name");

    if (strcmp(attrib, "battery") == 0) {
      if ((attrib = configuration_attrib (item, "interval")) != NULL)
        battery->interval = atoi(attrib);

      if ((attrib = configuration_attrib (item, "critical")) != NULL)
        battery->critical = atoi(attrib);

      if ((attrib = configuration_attrib (item, "warning")) != NULL)
        battery->warning = atoi(attrib);

      break;
    }
    chain = configuration_find_next (chain, "widget");
  }

  /* Configuration for the clock applet. */
  clock->interval = 60;	 /* clock update interval fallback value */
  chain = configuration_find (config, "modules");

  while ((item = configuration_find (chain, "widget")) != NULL) {
    attrib = configuration_attrib (item, "name");

    if (strcmp(attrib, "clock") == 0) {
      if ((attrib = configuration_attrib (item, "format")) != NULL)
        clock->format = attrib;

      if ((attrib = configuration_attrib (item, "interval")) != NULL)
        clock->interval = atoi(attrib);

      break;
    }
    chain = configuration_find_next (chain, "widget");
  }

  if (clock->format == NULL)
    clock->format = g_strdup ("%H:%M");

  /* Configuration of remaining settings. */
  self->icons   = icons;
  self->splash  = splash;
  self->battery = battery;
  self->clock   = clock;

  self->indent = 0;		/* default indents at each end */
  self->margin = 0;		/* default margins at each end */
  self->thickness = 32;		/* default panel bar thickness */
  self->place = GTK_POS_BOTTOM;	/* default place & orientation */

  if ((item = configuration_find (config, "indent")) != NULL)
    self->indent = atoi(item->element);

  if ((item = configuration_find (config, "margin")) != NULL)
    self->margin = atoi(item->element);

  if ((item = configuration_find (config, "thickness")) != NULL)
    self->thickness = atoi(item->element);

  if ((item = configuration_find (config, "place")) != NULL) {
    if (strcmp(item->element, "TOP") == 0)
      self->place = GTK_POS_TOP;
    else if (strcmp(item->element, "BOTTOM") == 0)
      self->place = GTK_POS_BOTTOM;
    else if (strcmp(item->element, "RIGHT") == 0)
      self->place = GTK_POS_RIGHT;
    else if (strcmp(item->element, "LEFT") == 0)
      self->place = GTK_POS_LEFT;
  }
  panel_config_orientation (self);  /* need to config orientation */
} /* </panel_config_settings> */

/*
 * (private) panel_config_moduli
 */
static GList *
panel_config_moduli (GList *list, ConfigurationNode *config)
{
  GList *iter;
  GList *moduli = NULL;

  ConfigurationNode *chain;
  ConfigurationNode *item;
  Modulus *applet;
  gchar *name;

  /* Iterate through widgets listed in the configuration. */
  chain = configuration_find (config, "modules");

  while (chain->depth > config->depth) {
    if (strcmp(chain->element, "widget") == 0) {
      name = configuration_attrib (chain, "name");

      /* Iterate through the plugin modules for the widget. */
      for (iter = list; iter != NULL; iter = iter->next) {
        applet = (Modulus *)iter->data;

        /* <widget name="{value}"> needs to match applet->name */
        if (name == NULL || strcmp(applet->name, name) != 0)
          continue;

        if ((item = configuration_find (chain, "icon")) != NULL)
          applet->icon = item->element;

        if ((item = configuration_find (chain, "text")) != NULL)
          applet->description = item->element;

        if ((item = configuration_find (chain, "place")) != NULL) {
          if (strcmp(item->element, "SCREEN") == 0)
            applet->place = SCREEN;
          else if (strcmp(item->element, "CONTAINER") == 0)
            applet->place = ADD_TO_CONTAINER;
          else if (strcmp(item->element, "START") == 0)
            applet->place = PACK_START;
          else if (strcmp(item->element, "END") == 0)
            applet->place = PACK_END;
          else
            applet->place = NONE;
        }
        moduli = g_list_append (moduli, applet);
      }
    }
    chain = chain->next;
  }
  return moduli;
} /* </panel_config_moduli> */

/*
 * (private) plugin_widgets
 */
static GList *
plugin_widgets (GList *list, GlobalPanel *self, guint space)
{
  GList *moduli = list;
  ConfigurationNode *config = self->config;
  ConfigurationNode *chain = configuration_find (config, "modules");
  ConfigurationNode *item;
  gchar *path;

  if (chain == NULL)	/* <modules> not in configuration */
    return moduli;

  if ((path = configuration_path (chain)) != NULL) {
    moduli = moduli_space (moduli, path, space, self);

    while (path != NULL) {
      item = configuration_find (chain, "path");  /* next <path> */

      if (item->next && (item->next)->next) {
        chain = (item->next)->next;

        if ((path = configuration_path (chain)) != NULL)
          moduli = moduli_space (moduli, path, space, self);
      }
    }
  }
  return moduli;
} /* </plugin_widgets> */

/*
 * interface - construct user interface
 */
GtkWidget *
interface (GlobalPanel *self)
{
  GtkWidget *frame;		/* user interface frame */
  GtkWidget *layout;		/* user interface layout */
  GtkWidget *space;		/* margin space */

  Modulus *applet;		/* plugin modules */
  GList *iter;

  if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
    layout = gtk_hbox_new (FALSE, 1);
  else
    layout = gtk_vbox_new (FALSE, 1);

  /* Spacing at both ends of the layout box. */
  space = gtk_label_new ("");
  panel_update_pack_position(self, PACK_START, space, 0);
  gtk_box_pack_start (GTK_BOX(layout), space, FALSE, FALSE, self->indent);
  gtk_widget_show (space);

  space = gtk_label_new ("");
  panel_update_pack_position(self, PACK_END, space, 0);
  gtk_box_pack_end (GTK_BOX(layout), space, FALSE, FALSE, self->indent);
  gtk_widget_show (space);

  /* Give the layout box a nicer look. */
  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_OUT);

  /* Contruct the user interface. */
  gtk_container_add (GTK_CONTAINER(frame), layout);
  gtk_widget_show (layout);

  /* Activate all modules from self->moduli */
  for (iter = self->moduli; iter != NULL; iter = iter->next) {
    applet = (Modulus *)iter->data;

    if (applet->module_open)
      applet->module_open (applet, self);

    if (applet->widget) {
      if (applet->place == ADD_TO_CONTAINER)
        gtk_container_add (GTK_CONTAINER(layout), applet->widget);  
      else if (applet->place == PACK_START)
        gtk_box_pack_start (GTK_BOX(layout), applet->widget, FALSE, FALSE, 0);
      else if (applet->place == PACK_END)
        gtk_box_pack_end (GTK_BOX(layout), applet->widget, FALSE, FALSE, 0);

      if (applet->place != NONE)       /* additional check */
        gtk_widget_show (applet->widget);
    }
  }

  /* Add gtk_event_box_new() to the interface layout. */
  menu_dispatch (self, layout);

  /* Add desktop icon widgets. */
  if (configuration_find (self->config, "desktop") != NULL)
    desktop_config (self);

  return frame;
} /* </interface> */

/*
 * initialize
 */
void
initialize (GlobalPanel *self)
{
  GList *moduli;
  gchar resource[FILENAME_MAX];
  gchar *home = getenv("HOME");

  self->config = NULL;
  self->resource = g_strdup_printf("%s/.config/panel", home);

  /* Read (and cache) user (or system-wide) configuration file. */
  if (access(self->resource, R_OK) == 0) {
    self->config = configuration_read (self->resource, Schema, FALSE);
  }
  else {
    if (access("/usr/local/share/panel/config", R_OK) == 0)
      strcpy(resource, "/usr/local/share/panel/config");
    else
      strcpy(resource, "/usr/share/panel/config");

    if (access(resource, R_OK) == 0) {
      if ((self->config = configuration_read (resource, Schema, FALSE))) {
        FILE *stream = fopen(self->resource, "w");

        if (stream) {
          configuration_write (self->config, ConfigurationHeader, stream);
          fclose(stream);
        }
      }
    }
    else {	/* write default configuration resource file */
      int config;
      char *scan;

      strcpy(resource, self->resource);   /* obtain parent directory path */
      if ( (scan = strrchr(resource, '/')) )
        *scan = (char)0;

      if (access(resource, F_OK) != 0) /* create parent directory as needed */
        mkdir(resource, 0755);

      if ((config = open(self->resource, O_CREAT | O_WRONLY, 0640)) > 0) {
        write(config, ConfigurationHeader, strlen(ConfigurationHeader));
        write(config, DefaultConfiguration, strlen(DefaultConfiguration));
        write(config, "</", 2);
        write(config, Schema, strlen(Schema));
        write(config, ">\n", 2);
        close(config);

        self->config = configuration_read (self->resource, Schema, FALSE);
      }
    }
  }

  if (self->config) {  /* start screaming if self->config == NULL */
    debug_configuration_cache (self->config, stdout);

    panel_config_settings (self);	/* initial settings fron config */
    moduli = builtin_widgets (self);	/* add built-in modules widgets */

    /* Add plugin modules widgets. */
    moduli = plugin_widgets (moduli, self, MODULI_SPACE_TASKBAR);

    /* Arrange plugin modules according to configuration settings. */
    self->moduli = panel_config_moduli (moduli, self->config);
    g_list_free (moduli);

    /* $HOME/.trash is needed by the Recycle Bin Docklet */
    sprintf(resource, "%s/.trash", home);
    if (access(resource, F_OK) != 0)
      mkdir(resource, 0755);
  }
  global = self;   /* global program data structure */
} /* </initialize> */

/*
 * gpanel - main application constructor
 */
int
gpanel (GlobalPanel *panel)
{
  GtkWidget *frame;
  GtkWidget *widget;
  GList     *iter;

  /* Initialize program data structure. */
  initialize (panel);

  if (panel->config == NULL)
    return EX_CONFIG;

  /* Create a new top level window and set attributes. */
  widget = panel->window = sticky_window_new (GDK_WINDOW_TYPE_HINT_DOCK,
                                              panel->width, panel->height,
                                              panel->xpos, panel->ypos);
  gtk_widget_show (widget);

  /* Contruct the user interface. */
  frame = interface (panel);
  gtk_container_add (GTK_CONTAINER(widget), frame);
  gtk_widget_show (frame);

  /* Iterate the alert notices list. */
  for (iter = panel->notice; iter != NULL; iter = iter->next)
    notice_at(100, 100, ICON_WARNING, "%s: %s.", Program, (gchar*)iter->data);

  g_list_free (panel->notice);
  panel->notice = NULL;

  return EX_OK;
} /* </gpanel> */

/*
 * main - main program implementation
 */
int
main(int argc, char *argv[])
{
  struct _GlobalPanel memory;	/* see, gpanel.h */

  FILE *stream;			/* file descriptor for lockfile */
  gchar *lockfile;
  int status;
  int idx;

  Program = basename(argv[0]);
  Release = "0.9.2";

  for (idx = 1; idx < argc; idx++)
    if (strcmp(argv[idx], "-V") == 0) {
      printf("\n%s %s %s\n", Program, Release, Information);
      exit(1);
    }
    else if (strcmp(argv[idx], "-d") == 0) {
      printf("\n%s: running in DEBUG mode.\n\n", Program);
      debug = TRUE;
    }

  gtk_set_locale ();		 /* initialization of the GTK */
  gtk_init (&argc, &argv);

  /* There can only be one.. instance of this program running. */
  lockfile = memory.lockfile = g_strdup_printf("/tmp/.%s-lock", Program);
  status = EX_OK;

  if ((stream = fopen(lockfile, "r"))) {  /* lockfile exists */
    int bytes = sizeof(double);

    char pid[bytes];
    fgets(pid, bytes, stream);

    if (strlen(pid) > 0) {
      pid[strlen(pid) - 1] = (char)0;	/* chomp newline */

      if (access(g_strdup_printf("/proc/%s", pid), F_OK) == 0)
        status = EX_UNAVAILABLE;
    }
    fclose(stream);
  }

  if (status == EX_OK) {
    status = gpanel (&memory);	 /* main application */

    if ((stream = fopen(lockfile, "w"))) {
      fprintf(stream, "%d\n", getpid());
      fclose(stream);
    }
  }

  switch (status) {
    case EX_OK:
      gtk_main ();		/* main event loop */
      break;
    case EX_UNAVAILABLE:
      notice(NULL, ICON_WARNING, "%s: %s.", Program, _("is already running"));
      break;
    case EX_CONFIG:
      notice(NULL, ICON_ERROR, "%s: %s.", Program,
                                     _("cannot find configuration file"));
      break;
    default:
      notice(NULL, ICON_ERROR, "%s: %s.", Program,
                                     _("internal program error"));
  }
  return status;
} /* </main> */
