/* $Id: gpanel.h,v 1.16 2007/09/30 15:41:11 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#ifndef GPANEL_H
#define GPANEL_H

#include <sysexits.h>

#define WNCK_I_KNOW_THIS_IS_UNSTABLE
#include <libwnck/libwnck.h>

#include "xmlconfig.h"
#include "module.h"
#include "window.h"
#include "dialog.h"
#include "docklet.h"

G_BEGIN_DECLS

/* Desktop shortcut actions. */
typedef enum _desktop_actions DesktopAction;

enum _desktop_actions {
  DESKTOP_SHORTCUT_CREATE,
  DESKTOP_SHORTCUT_DELETE,
  DESKTOP_SHORTCUT_EDIT,
  DESKTOP_SHORTCUT_OPEN
};

/* Global program data structure */
typedef struct _GlobalPanel GlobalPanel;

typedef struct _PanelBattery PanelBattery;
typedef struct _PanelClock   PanelClock;
typedef struct _PanelDesktop PanelDesktop;
typedef struct _PanelIcons   PanelIcons;
typedef struct _PanelMenu    PanelMenu;
typedef struct _PanelSplash  PanelSplash;
typedef struct _PanelSetting PanelSetting;
typedef struct _PanelWnck    PanelWnck;

struct _GlobalPanel
{
  gchar *lockfile;		/* program lock file */
  gchar *resource;		/* configuration file */

  ConfigurationNode *config;	/* configuration cache */

  PanelDesktop *desktop;	/* create/edit desktop shortcut */
  PanelSetting *settings;	/* edit configuration settings */

  PanelBattery *battery;	/* battery applet configuration */
  PanelClock   *clock;		/* clock applet configuration */
  PanelIcons   *icons;		/* icons configuration data */
  PanelMenu    *menu;		/* menu configuration data */
  PanelSplash  *splash;		/* splash screen configuration */
  PanelWnck    *wnck;		/* wnck configuration data */

  GList *moduli;		/* plugin modules list */
  GList *notice;		/* alert notices list */
  GList *path;			/* PATH environment */

  GtkWidget *window;		/* main window widget */
  GtkWidget *options;		/* start menu options */

  GtkOrientation orientation;	/* GTK_ORIENTATION_{HORIZONTAL, VERTICAL} */
  GtkPositionType place;	/* GTK_POS_{TOP, BOTTOM, LEFT, RIGHT} */
  guint thickness;		/* thickness of the panel bar */
  guint margin;			/* margin at each end */
  guint indent;			/* indent at each end */

  gint height, width;	 	/* panel bar main window dimensions */
  gint xpos, ypos;		/* (x,y) main window coordinates */
  gint spos, epos;		/* start and end pack position */
};

struct _PanelBattery {
  GlobalPanel *panel;		/* parent reference */

  GdkPixbuf *chargingIcon;	/* pixbuf for the charging icon */
  GdkPixbuf *dischargingIcon;	/* pixbuf for the discharing icon */
  GtkWidget *stateImage;	/* image shown on the panel */
  GtkWidget *stateText;		/* text shown on the panel */

  guint interval;		/* update interval in seconds */
  guint warning;		/* warning threshold, percent */
  guint critical;		/* critical threshold, percent */
};

struct _PanelClock {
  GtkWidget *display;		/* display drawing area */
  gchar     *format;		/* time display format (strftime(3)) */
  guint     interval;		/* update interval in seconds */
};

struct _PanelDesktop
{
  ConfigurationNode* node;	/* pertinent configuration node */
  FileChooser *chooser;		/* file chooser for selecting icon */

  DesktopAction action;		/* action to perform on desktop */
  Docklet *shortcut;		/* screen representation */

  GtkWidget *menu;		/* builtin actions menu */
  GtkWidget *window;		/* popup widget window */
  GtkWidget *iconview;		/* icon display widget */
  GtkWidget *filer;		/* file chooser layout */

  GdkPixbuf *render;		/* pixbuf of the current icon */
  GtkWidget *canvas;		/* icon preview drawing area */
  GtkWidget *name;		/* name text entry field */
  GtkWidget *exec;		/* exec text entry field */
  GtkWidget *hint;		/* action hints */

  const gchar *icon;		/* default icon image pathname */
  gboolean activate;		/* governs if to activate or not */
  gint iconsize;		/* fixed icon width and height */
};

struct _PanelIcons
{
  GList *path;			/* list of directory paths to search */
  guint size;			/* width and height of each icon */
};

struct _PanelMenu
{
  GtkWidget *options;		/* menu options widget */
  guint iconsize;		/* width and height of each icon */
};

struct _PanelSplash
{
  guint expire;			/* timeout in seconds (0 => disable) */
  const gchar *fontdesc;	/* PangoFontDescription to use */
  GdkColor *fg;			/* foreground color */
  GdkColor *bg;			/* background color */
};

struct _PanelSetting
{
  GtkWidget *window;		/* popup widget window */
  gboolean activate;		/* governs if to activate or not */
};

struct _PanelWnck
{
  GtkWidget  *pager;		/* pager widget */
  GtkWidget  *tasklist;		/* tasklist widget */
  WnckScreen *screen;		/* see, libwnck/screen.h */
  WnckTasklistGroupingType grouping; /* see, libwnck/tasklist.h */
  gboolean   allspaces;		/* tasklist include all workspaces */
  gboolean   showdesktop;	/* showing desktop toggle */
  guint      rows;		/* pager number of rows */
};

/* Methods exported by implementation. */
void debug_configuration_cache (ConfigurationNode *config, FILE* stream);

void activate (GtkWidget *button, GdkEventButton *event, PanelMenu *menu);
void executer (GtkWidget *widget, ConfigurationNode *node);
void logout (GtkWidget *widget, ConfigurationNode *node);
void restart (GtkWidget *widget, ConfigurationNode *node);

GtkWidget *menu_config (Modulus *applet, GlobalPanel *self);
GtkWidget *menu_options_config (ConfigurationNode *node, GlobalPanel *self);
GtkWidget *menu_submenu_config (ConfigurationNode *node, GlobalPanel *self);
GtkWidget *menu_showdesktop (Modulus *applet, GlobalPanel *self);
void menu_dispatch (GlobalPanel *self, GtkWidget *layout);

void desktop_config (GlobalPanel *self);
void desktop_menu (ConfigurationNode *node, Docklet *docklet);
void desktop_settings (ConfigurationNode *node, DesktopAction act);

Docklet *desktop_shortcut (GlobalPanel *self,
                           ConfigurationNode *node,
                           gint width, gint height,
                           gint xpos, gint ypos,
                           const gchar *icon,
                           const gchar *text,
                           const gchar *font,
                           GdkColor *fg,
                           GdkColor *bg);

void panel_config_orientation (GlobalPanel *self);
void panel_update_pack_position (GlobalPanel *self,
                                 ModulusPlace place,   
                                 GtkWidget *label,
                                 gint iconsize);

void settings_popup (GtkWidget *widget, ConfigurationNode *node);
void spawn (ConfigurationNode *node, GlobalPanel *self);

int gpanel (GlobalPanel *self);

G_END_DECLS

#endif /* </GPANEL_H */
