/* $Id: menu.c,v 1.12 2007/09/30 15:41:11 mgdepalma Exp $
 *
 * Copyright (C) 2007 Softcraft Impresa, LLC <info@softcraft.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; You may only use version 2 of the License,
 * you have no option to use any other version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include "gpanel.h"
#include "language.h"

extern const char *Program, *Release, *Information;	/* see, gpanel.c */
extern const char *ConfigurationHeader, *Schema;        /* .. */
extern gboolean debug;					/* .. */

static char *SettingsMenu =
"<?xml version=\"1.0\" encoding=\"utf-16\"?>\n"
"<menu name=\"PanelSettings\" iconsize=\"16\">\n"
" <options>\n"
"  <item name=\"Configuration\">\n"
"   <icon>settings.png</icon>\n"
"   <exec type=\"method\">settings</exec>\n"
"  </item>\n"
"  <item name=\"Create Shortcut\">\n"
"   <icon>wizard.png</icon>\n"
"   <exec type=\"method\">shortcut:create</exec>\n"
"  </item>\n"
"  <separator />\n"
"  <item name=\"About\">\n"
"   <icon>info.png</icon>\n"
"   <exec type=\"method\">about</exec>\n"
"  </item>\n"
" </options>\n"
"</menu>\n";

/*
 * (private) about
 */
static gboolean
about (GtkWidget *widget, ConfigurationNode* node)
{
  GlobalPanel *global = node->data;

  notice_at(100, 100, ICON_SNAPSHOT,
               "\n%s %s %s", Program, Release, Information);
  return TRUE;
} /* </about> */

/*
 * clean application exit
 */
static gboolean
finis (GtkWidget *widget, ConfigurationNode *node)
{
  GlobalPanel *global = node->data;
  gchar *resource = global->resource;

  if (rename(resource, g_strdup_printf("%s.old", resource)) == 0) {
    FILE *stream = fopen(resource, "w");

    if (stream) {
      configuration_write (global->config, ConfigurationHeader, stream);
      fclose(stream);
    }
    else {
      fprintf(stderr, "%s: cannot open %s for writing\n", Program, resource);
    }
  }
  else {
    fprintf(stderr, "%s: cannot backup %s, will not write over\n",
                      Program, resource);
  }
  gtk_main_quit ();	/* TODO ....more.. */

  return TRUE;
} /* </finis> */

/*
 * (protected) activate
 */
void
activate (GtkWidget *button, GdkEventButton *event, PanelMenu *menu)
{
  if (event->button == 1) {
    gtk_menu_popup (GTK_MENU (menu->options), NULL, NULL, NULL, NULL,
                      event->button, event->time);
  }
} /* <activate> */

/*
 * item_check_access
 */
gchar * 
item_check_access (ConfigurationNode *node, GlobalPanel *panel, gchar *attr)
{
  static gchar file[FILENAME_MAX];
  static gchar args[FILENAME_MAX];

  gchar *value  = (attr) ? configuration_attrib (node, attr) : node->element;
  gchar *result = NULL;
  
  if (value != NULL) {		/* check access based on given attribute */
    gchar *scan;

    strcpy(file, value);	/* scan for the first space character */
    strcpy(args, "");

    if ((scan = strchr(file, ' '))) {
      strcpy(args, (scan+1));
      *scan = (char)0;
    }

    if (file[0] == '/') {	/* absolute path specified */
      if (access(file, X_OK) == 0) {
        strcpy(file, value);
        result = file;
      }
    }
    else {
      const gchar *pathname = file_path_find (panel->path, file);
      if (pathname != NULL) {
        sprintf(file, "%s %s", pathname, args);
        result = file;
      }
    }
  }
  else {			/* pass, attribute was not specified */
    result = node->element;
  }
  return result;
} /* </item_check_access> */

/*
 * (protected) spawn
 */
void
spawn (ConfigurationNode *node, GlobalPanel *panel)
{
  gchar *file;
  gchar command[FILENAME_MAX];

  if ((file = item_check_access (node, panel, NULL))) {
    sprintf(command, "%s &", file);
    system (command);
  }
  else {
    notice_at(100, 100, ICON_WARNING, "[%s]%s: %s.",
               Program, node->element, _("command not found"));
  }
} /* </spawn> */

/*
 * (protected) executer
 */
void
executer (GtkWidget *widget, ConfigurationNode *node)
{
  ConfigurationNode *exec = configuration_find (node, "exec");
  gchar *type = configuration_attrib (exec->back, "type");

  if (debug) {
    gchar *name = configuration_attrib (node, "name");
    printf("[executer]%s.exec: %s (type=%s)\n", name, exec->element, type);
  }

  if (type != NULL && strcmp(type, "method") == 0) {
    if (strcmp(exec->element, "about") == 0)
      about (widget, node);
    else if (strcmp(exec->element, "exit") == 0)
      finis (widget, node);
    else if (strcmp(exec->element, "logout") == 0)
      logout (widget, node);
    else if (strcmp(exec->element, "restart") == 0)
      restart (widget, node);
    else if (strcmp(exec->element, "settings") == 0)
      settings_popup (widget, node);
    else if (strcmp(exec->element, "shortcut:create") == 0)
      desktop_settings (node, DESKTOP_SHORTCUT_CREATE);
    else if (strcmp(exec->element, "shortcut:delete") == 0)
      desktop_settings (node, DESKTOP_SHORTCUT_DELETE);
    else if (strcmp(exec->element, "shortcut:edit") == 0)
      desktop_settings (node, DESKTOP_SHORTCUT_EDIT);
    else if (strcmp(exec->element, "shortcut:open") == 0)
      desktop_settings (node, DESKTOP_SHORTCUT_OPEN);
    else
      notice_at(100, 100, ICON_ERROR, "[%s]%s: %s.",
                 Program, exec->element, _("unimplemented method"));
  }
  else {
    spawn (exec, node->data);
  }
} /* </executer> */

/*
 * logout - dialog to end session
 */
void
logout (GtkWidget *widget, ConfigurationNode *node)
{
  GlobalPanel *panel = node->data;
  remove(panel->lockfile);
  finis (widget, node);
} /* </logout> */

/*
 * (protected) restart
 */
void
restart (GtkWidget *widget, ConfigurationNode *node)
{
  struct _GlobalPanel memory;	/* see, gpanel.h */
  GlobalPanel *panel = node->data;
#if 0
  GList *iter;

  for (iter = panel->moduli; iter != NULL; iter->next)
    MODULE_CLOSE
#endif

  finis (panel->window, node);
  gtk_widget_destroy (panel->window);
  gpanel (&memory);

  gtk_main ();
} /* </restart> UNSTABLE */

/*
 * (private) options
 */
static void
menu_options (GtkWidget *widget, GdkEventButton *event, GlobalPanel *self)
{
  g_return_if_fail (GTK_IS_WIDGET (widget));

  if (event->button == 3)
    gtk_menu_popup (GTK_MENU (self->options), NULL, NULL, NULL, NULL,
                      event->button, event->time);
} /* </menu_options> */

/*
 * settings_popup - activate popup window for editing configuration settings
 */
void
settings_popup (GtkWidget *widget, ConfigurationNode *node)
{
  GlobalPanel *self = node->data;   /* cannot be NULL */
  PanelSetting *settings = self->settings;
  
  if (settings->activate) {
    gtk_widget_show (settings->window);
    settings->activate = FALSE;
  }
} /* </settings_popup> */

/*
 * settings_dismiss
 */
static gboolean
settings_dismiss (GtkWidget *button, GlobalPanel *self)
{
  PanelSetting *settings = self->settings;
  gtk_widget_hide (settings->window);
  settings->activate = TRUE;
  return TRUE;
} /* </settings_dismiss> */

/*
 * settings_new - popup window for editing configuration settings
 */
PanelSetting *
settings_new (GlobalPanel *self)
{
  GtkWidget *window, *frame, *layout, *layer;
  GtkWidget *button, *label;

  PanelSetting *settings = g_new0(PanelSetting, 1);
  settings->activate = TRUE;

  /* Create a popup window with a frame. */
  window = settings->window = sticky_window_new (GDK_WINDOW_TYPE_HINT_NORMAL,
                                                        280, 80, 100, 100);
  gtk_window_set_keep_above (GTK_WINDOW(window), TRUE);

  frame = gtk_frame_new (NULL);
  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_OUT);
  gtk_container_add (GTK_CONTAINER(window), frame);
  gtk_widget_show (frame);

  /* Create the inside window layout. */
  layout = gtk_vbox_new (FALSE, 1);
  gtk_container_add (GTK_CONTAINER(frame), layout);
  gtk_widget_show (layout);

  layer = gtk_hbox_new (FALSE, 1);
  gtk_box_pack_start (GTK_BOX(layout), layer, FALSE, FALSE, 2);
  gtk_widget_show (layer);

  label = gtk_label_new (_("[settings_new]not yet implemented"));
  gtk_box_pack_start (GTK_BOX(layer), label, FALSE, FALSE, 2);
  gtk_widget_show (label);

  /* Add the APPLY and CANCEL buttons. */
  layer = gtk_hbox_new (FALSE, 1);
  gtk_box_pack_start (GTK_BOX(layout), layer, FALSE, FALSE, 0);
  gtk_widget_show (layer);

  button = xpm_button(ICON_SNAPSHOT, _L(LABEL_OK));
  gtk_box_pack_end (GTK_BOX(layer), button, FALSE, FALSE, 0);
  gtk_button_set_relief (GTK_BUTTON(button), GTK_RELIEF_NONE);

  g_signal_connect (G_OBJECT(button), "clicked",
                     G_CALLBACK(settings_dismiss), self);
  gtk_widget_show (button);

  return settings;
} /* settings_new */

/*
 * menu_item_config
 */
GtkWidget *
menu_item_config (ConfigurationNode *config, GlobalPanel *self)
{
  ConfigurationNode *node;
  GtkWidget *image, *item;
  PanelIcons *icons = self->icons;
  gchar *name = configuration_attrib (config, "name");
  guint iconsize = (self->menu)->iconsize;

  if (name == NULL)	/* shouldn't happen.. */
    return NULL;

  if (item_check_access (config, self, "check") == NULL)
    return NULL;

  item = gtk_image_menu_item_new_with_label (_(name));
  g_signal_connect (item, "activate", G_CALLBACK (executer), config);
  config->data = self;	/* pass global program attributes */

  if ((node = configuration_find (config, "icon")) != NULL) {
    name = (gchar *)file_path_find (icons->path, node->element);

    if (name != NULL) {
      image = image_new_from_file_scaled (name, iconsize, iconsize);
      gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    }
  }
  return item;
} /* </menu_item_config> */

/*
 * menu_element_config
 *
 * parse a menu element which can be:
 *   <item>, <separator>, or <submenu>
 */
GtkWidget *
menu_element_config (ConfigurationNode *node, GlobalPanel *self)
{
  GdkPixbuf *pixbuf;
  GtkWidget *image, *submenu;
  GtkWidget *item = NULL;
  PanelIcons *icons = self->icons;
  guint iconsize = (self->menu)->iconsize;
  gchar *name;

  if (item_check_access (node, self, "check") == NULL)
    return NULL;

  if (strcmp(node->element, "item") == 0) {
    item = menu_item_config (node, self);
  }
  else if (strcmp(node->element, "separator") == 0) {
    item = gtk_separator_menu_item_new();
  }
  else if (strcmp(node->element, "submenu") == 0) {
    if ((name = configuration_attrib (node, "icon")) != NULL)
      name = (gchar *)file_path_find (icons->path, name);

    if (name != NULL) {
      image = image_new_from_file_scaled (name, iconsize, iconsize);
    }
    else {
      pixbuf = xpm_pixbuf_scale (ICON_DIRS, iconsize, iconsize, NULL);
      image = gtk_image_new_from_pixbuf (pixbuf);
    }

    name = configuration_attrib (node, "name");
    item = gtk_image_menu_item_new_with_label (_(name));
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);

    submenu = menu_submenu_config (node, self);
    gtk_menu_item_set_submenu (GTK_MENU_ITEM (item), submenu);
  }
  return item;
} /* </menu_element_config> */

/*
 * menu_submenu_config
 */
GtkWidget *
menu_submenu_config (ConfigurationNode *chain, GlobalPanel *self)
{
  ConfigurationNode *node;
  GtkWidget *item, *options = gtk_menu_new ();
  guint depth = chain->depth + 1;

  for (node = chain->next; node != NULL; node = node->next) {
    if (node->depth == depth && node->type != XML_READER_TYPE_END_ELEMENT) {
      if ((item = menu_element_config (node, self)) != NULL) {
        gtk_menu_shell_append (GTK_MENU_SHELL(options), item);
        gtk_widget_show (item);
      }
    }
    else if (node->depth == chain->depth)  /* same depth.. move on */
      break;
  }
  return options;
} /* </menu_submenu_config> */

/*
 * menu_options_config
 */
GtkWidget *
menu_options_config (ConfigurationNode *config, GlobalPanel *self)
{
  ConfigurationNode *chain  = configuration_find (config, "options");
  GtkWidget *options = gtk_menu_new();
  GtkWidget *item;

  if (chain != NULL) {
    guint depth = chain->depth + 1;
    ConfigurationNode *node = chain->next;
    chain = configuration_find (chain, "/options");

    for (; node != chain && node != NULL; node = node->next) {
      if (node->depth == depth && node->type != XML_READER_TYPE_END_ELEMENT) {
        if ((item = menu_element_config (node, self)) != NULL) {
          gtk_menu_shell_append (GTK_MENU_SHELL(options), item);
          gtk_widget_show (item);
        }
      }
    }
  }
  else {		/* default fallback menu options */
    GdkPixbuf *pixbuf;
    GtkWidget *image;
    guint iconsize = (self->menu)->iconsize;

    pixbuf = xpm_pixbuf_scale (ICON_QUIT, iconsize, iconsize, NULL);
    image = gtk_image_new_from_pixbuf (pixbuf);

    item = gtk_image_menu_item_new_with_label (_L(LABEL_CLOSE));
    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
    g_signal_connect (item, "activate", G_CALLBACK (logout), NULL);

    gtk_menu_shell_append (GTK_MENU_SHELL(options), item);
    gtk_widget_show (item);
  }
  return options;
} /* </menu_options_config> */

/*
 * menu_config
 */
GtkWidget *
menu_config (Modulus *applet, GlobalPanel *self)
{
  ConfigurationNode *config = configuration_find (self->config, "menu");
  GtkWidget *button, *image, *label, *layout;
  PanelIcons *icons = self->icons;
  PanelMenu *menu;
  guint iconsize;
  gchar *value;

  if (config == NULL)	/* <menu ..> not in the configuration */
    return NULL;

  /* Create the main widget visible on the panel bar. */
  if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
    layout = gtk_hbox_new (FALSE, 0);
  else
    layout = gtk_vbox_new (FALSE, 0);

  gtk_container_set_border_width (GTK_CONTAINER (layout), 2);
  gtk_widget_show (layout);

  /* prefer <menu><iconsize> over <panel><icons><size> */
  if ((value = configuration_attrib (config, "iconsize")) != NULL)
    iconsize = atoi(value);
  else
    iconsize = icons->size;

  /* see if an icon is specified in the configuration */
  if ((applet->icon = configuration_attrib (config, "icon")) != NULL) {
    const gchar *file = file_path_find (icons->path, applet->icon);
    if (file != NULL) {
      image = image_new_from_file_scaled (file, iconsize, iconsize);
      gtk_box_pack_start (GTK_BOX (layout), image, FALSE, FALSE, 3);
      gtk_widget_show (image);
    }
  }

  /* use the name attribute in the configuration */
  label = gtk_label_new (_(configuration_attrib (config, "name")));
  gtk_box_pack_start (GTK_BOX (layout), label, FALSE, TRUE, 3);
  gtk_widget_show (label);

  /* adjust packing position */
  panel_update_pack_position(self, applet->place, label, iconsize);

  /* build struct _panel_menu */
  menu = self->menu = g_new0(PanelMenu, 1);
  menu->iconsize = iconsize;	/* used in menu_options_config() */
  menu->options  = menu_options_config (config, self);

  /* main widget for the menu */
  button = gtk_toggle_button_new ();
  gtk_container_add (GTK_CONTAINER (button), layout);
  gtk_button_set_relief (GTK_BUTTON (button), GTK_RELIEF_NORMAL);
  gtk_signal_connect (GTK_OBJECT (button), "button_press_event",
                       GTK_SIGNAL_FUNC (activate), menu); 

  return button;
} /* </menu_config> */

/*
 * menu_dispatch - add settings options menu
 */
void
menu_dispatch (GlobalPanel *self, GtkWidget *layout)
{
  ConfigurationNode *config;
  GtkWidget *dispatch = gtk_event_box_new ();

  /* Construct settings popup menu. */
  config = configuration_read (SettingsMenu, NULL, TRUE);
  self->options = menu_options_config (config, self);

  /* Add gtk_event_box_new() to catch "button-press-event" on panel bar. */
  gtk_container_add (GTK_CONTAINER(layout), dispatch);
  g_signal_connect (G_OBJECT(dispatch), "button-press-event",
                             G_CALLBACK(menu_options), self);
  gtk_widget_show (dispatch);

  /* Construct _struct PanelSetting for configuration settings. */
  self->settings = settings_new (self);
} /* </menu_dispatch> */
